@@ -30,6 +30,7 @@
 #include <fb303/ExportType.h>
 #include <fb303/SimpleLRUMap.h>
 #include <fb303/ThreadLocalStatsMap.h>
+#include <folly/ConcurrentLazy.h>
 #include <folly/Format.h>
 #include <folly/MapUtil.h>
 #include <folly/Overload.h>
@@ -476,20 +477,30 @@ class FormattedKeyHolder {
       (std::is_convertible_v<T, folly::StringPiece> || std::is_integral_v<T>) &&
       !std::is_same_v<T, std::nullptr_t> && !std::is_same_v<T, bool>;
 
-  // Hash function
-  struct SubkeyHash //
-      : private folly::hasher<int64_t>,
-        private folly::hasher<std::string>,
-        private folly::hasher<std::string_view> {
-    using is_transparent = void;
-    using folly_is_avalanching = folly::Conjunction<
-        folly::hasher<int64_t>::folly_is_avalanching,
-        folly::hasher<std::string>::folly_is_avalanching,
-        folly::hasher<std::string_view>::folly_is_avalanching>;
+  struct string_view_hasher_murmur64 {
+    using folly_is_avalanching = std::true_type;
 
-    using folly::hasher<int64_t>::operator();
-    using folly::hasher<std::string>::operator();
-    using folly::hasher<std::string_view>::operator();
+    size_t operator()(std::string_view sv) const noexcept {
+      constexpr uint64_t kSeed = 0xc70f6907ULL; // seed used by libstdc++
+      return folly::hash::murmurHash64(sv.data(), sv.size(), kSeed);
+    }
+    size_t operator()(const std::string& s) const noexcept {
+      return operator()(std::string_view(s));
+    }
+  };
+
+  using int64_hasher = folly::hasher<int64_t>;
+  using string_view_hasher = string_view_hasher_murmur64;
+  static_assert(int64_hasher::folly_is_avalanching::value);
+  static_assert(string_view_hasher::folly_is_avalanching::value);
+
+  // Hash function
+  struct SubkeyHash : private int64_hasher, private string_view_hasher {
+    using is_transparent = void;
+    using folly_is_avalanching = std::true_type;
+
+    using int64_hasher::operator();
+    using string_view_hasher::operator();
 
     size_t operator()(const Subkey& v) const {
       return std::visit(*this, v);
@@ -875,21 +886,17 @@ class TimeseriesWrapper {
  private:
   std::string key_;
 
-  folly::ThreadLocal<std::shared_ptr<ThreadCachedServiceData::TLTimeseries>>
-      tlTimeseries_;
+  //  Holds pointers directly to TLTimeseries, and holds deleters which hold
+  //  copies of the owning shared_ptr objects.
+  folly::ThreadLocalPtr<ThreadCachedServiceData::TLTimeseries> tlTimeseries_;
 
-  inline ThreadCachedServiceData::TLTimeseries* tcTimeseries() {
-    ThreadCachedServiceData::TLTimeseries* cached = tlTimeseries_->get();
-    if (cached) {
-      return cached;
-    }
-
-    auto timeseries =
-        ThreadCachedServiceData::getStatsThreadLocal()->getTimeseriesSafe(key_);
-    *tlTimeseries_ = timeseries;
-    return timeseries.get();
+  FOLLY_ALWAYS_INLINE ThreadCachedServiceData::TLTimeseries* tcTimeseries() {
+    auto cached = tlTimeseries_.get();
+    return FOLLY_LIKELY(!!cached) ? cached : tcTimeseriesSlow();
   }
 
+  FOLLY_NOINLINE ThreadCachedServiceData::TLTimeseries* tcTimeseriesSlow();
+
   template <typename... Args>
   void exportStats(const ExportedStat* statPrototype, const Args&... args) {
     int _[] = {
@@ -1429,6 +1436,33 @@ class DynamicHistogramWrapper {
   const internal::HistogramSpec spec_;
 };
 
+/**
+ * Lazily create a timeseries on the first call to add() & co.
+ *
+ * Register it when it's actually used to reduce fb303 / ODS footprint when the
+ * timeseries is not used in a subset of instances of a service.
+ *
+ * Thread Safe: only one thread will construct (and register) the timeseries.
+ */
+template <typename TimeseriesT = TimeseriesWrapper>
+class LazyTimeseries {
+ public:
+  template <typename... Args>
+  explicit LazyTimeseries(const Args&... args)
+      : t_([=] { return TimeseriesT(args...); }) {}
+
+  void add(int64_t value = 1) {
+    t_().add(value);
+  }
+
+  void addAggregated(int64_t sum, int64_t numSamples) {
+    t_().addAggregated(sum, numSamples);
+  }
+
+ private:
+  folly::ConcurrentLazy<folly::Function<TimeseriesT(void)>> t_;
+};
+
 } // namespace facebook::fb303
 
 namespace facebook {
